@@ -652,6 +652,8 @@ gantt_model_get_column_type (GtkTreeModel *tree_model,
 		return MRP_TYPE_TASK;
 	case COL_COST:
 		return G_TYPE_LONG;	
+	case COL_COMPLETE:
+		return G_TYPE_INT;
 	default:
 		return G_TYPE_INVALID;
 	}
@@ -844,9 +846,11 @@ gantt_model_get_value (GtkTreeModel *tree_model,
 		t2 = mrp_task_get_latest_finish (task);
 		project = mrp_object_get_project (MRP_OBJECT (task));
 		
-		/* We don't support negative slack yet. */
+		/* We don't support negative slack */
 		if (t2 >= t1) {
-			duration = mrp_project_calculate_task_work (
+			/* use summary duration calc since we want to use 
+			 * the project calendar and ignore resource assignments */
+			duration = mrp_project_calculate_summary_duration (
 				project, task, t1, t2);
 		} else {
 			duration = 0;
@@ -888,6 +892,12 @@ gantt_model_get_value (GtkTreeModel *tree_model,
 
 		break;
 
+	case COL_COMPLETE:
+		g_value_init (value, G_TYPE_INT);
+		g_value_set_int (value, mrp_task_get_percent_complete (task));
+
+		break;
+
 	default:
 		g_warning ("Bad column %d requested", column);
 	}
