@@ -55,6 +55,8 @@
 
 #define MILESTONE_SIZE 5
 
+#define FUZZ  3
+
 /* Minimum width for a task to keep it visible. */
 #define MIN_WIDTH 2
 
@@ -97,10 +99,19 @@ typedef enum {
 	STATE_NONE          = 0,
 	STATE_DRAG_LINK     = 1 << 0,
 	STATE_DRAG_DURATION = 1 << 1,
+	STATE_DRAG_COMPLETE = 1 << 2,
 
-	STATE_DRAG_ANY = STATE_DRAG_LINK | STATE_DRAG_DURATION
+	STATE_DRAG_ANY = STATE_DRAG_LINK | STATE_DRAG_DURATION | STATE_DRAG_COMPLETE
 } State;
 
+typedef enum
+{
+	DRAG_NONE_SPOT,
+	DRAG_DURATION_SPOT,
+	DRAG_COMPLETE_SPOT,
+	DRAG_RELATION_SPOT
+} DragSpot;
+
 struct _PlannerGanttRowPriv {
 	/* FIXME: Don't need those per gantt row. */
 	GdkGC       *complete_gc;
@@ -151,6 +162,7 @@ struct _PlannerGanttRowPriv {
 	gdouble      bar_top; /* Top y position of the bar. */
 	gdouble      bar_bot; /* Bottom y position of the bar. */
 	gdouble      text_width;
+	gdouble      text_height;
 
 	/* Cached positions of each assigned resource. */
 	GArray      *resource_widths;
@@ -269,6 +281,17 @@ static gchar                 complete_stipple_pattern[] = { 0x02, 0x01 };
 static GdkBitmap            *break_stipple = NULL;
 static gchar                 break_stipple_pattern[] = { 0x03 };
 
+/* Data related to dragging items */
+static GnomeCanvasItem   *drag_item;
+static gdouble            drag_item_wx_min;
+static gdouble            drag_item_wx_max;
+static gdouble            drag_item_wy_min;
+static gdouble            drag_item_wy_max;
+static gdouble            drag_wx1, drag_wy1;
+static GnomeCanvasPoints *drag_points = NULL;
+static GnomeCanvasItem   *target_item;
+static GnomeCanvasItem   *old_target_item;
+
 GType
 planner_gantt_row_get_type (void)
 {
@@ -482,7 +505,7 @@ recalc_bounds (PlannerGanttRow *row)
 {
 	PlannerGanttRowPriv *priv;
 	GnomeCanvasItem     *item;
-	gint                 width;
+	gint                 width, height;
 	mrptime              t;
 	MrpTaskType          type;
 	gdouble              old_x, old_x_start, old_width;
@@ -500,13 +523,14 @@ recalc_bounds (PlannerGanttRow *row)
 
 	pango_layout_get_pixel_size (priv->layout,
 				     &width,
-				     NULL);
-		
+				     &height);
+
 	if (width > 0) {
 		width += TEXT_PADDING;
 	}
 
 	priv->text_width = width;
+	priv->text_height = height;
 
 	t = mrp_task_get_work_start (priv->task);
 	priv->x = t * priv->scale;
@@ -731,7 +755,7 @@ gantt_row_update_resources (PlannerGanttRow *row)
 	pango_layout_set_text (priv->layout, ", ", 2);
 	pango_layout_get_extents (priv->layout, NULL, &rect);
 	spacing = rect.width / PANGO_SCALE;
-	
+
 	x = 0;
 	resources = mrp_task_get_assigned_resources (priv->task);
 		
@@ -1944,14 +1968,15 @@ gantt_row_draw (GnomeCanvasItem *item,
 	rx2 = MIN (cx2 + TEXT_PADDING + priv->text_width, width);
 
 	if (priv->layout != NULL && rx1 < rx2) {
-		/* NOTE: cy1 - priv->bar_top: report to the top of the cell,
-		         + 3: is an empirical value to realign with task-tree result of the
-		         default gtkcellrenderertext ypad property (2) + 1 ??? ;). */
+		/* NOTE: cy1 is the pixel coordinate of the top of the bar. 
+			 subtract round(priv->bar_top) to bring us to the top of the row
+			 add priv->height / 2 to get to the center of the row
+			 subtract priv->text_height / 2 to get to the top of the text */
 		   
 		draw_cut_layout (drawable,
 				 GTK_WIDGET (item->canvas)->style->text_gc[GTK_STATE_NORMAL],
 				 cx2 + TEXT_PADDING,
-				 cy1 - priv->bar_top + 3, 
+				 cy1 - (gint)(priv->bar_top + 0.5) + (priv->height - priv->text_height) / 2,
 				 priv->layout);
 
 		if (priv->mouse_over_index != -1) {
@@ -2060,8 +2085,9 @@ gantt_row_notify_cb (MrpTask *task, GParamSpec *pspec, PlannerGanttRow *row)
 	 * it since it's a good optimization.
 	 */
 	else if (strcmp (pspec->name, "critical") != 0 &&
-			 strcmp (pspec->name, "priority") != 0 &&
-			 strcmp (pspec->name, "percent-complete")) {
+		 strcmp (pspec->name, "sched") != 0 &&
+		 strcmp (pspec->name, "priority") != 0 &&
+		 strcmp (pspec->name, "percent-complete")) {
 		return;
 	}
 	
@@ -2267,51 +2293,238 @@ gantt_row_disconnect_all_resources (MrpTask *task, PlannerGanttRow *row)
 	g_list_free (resources);
 }
 
-static gboolean
-gantt_row_scroll_timeout_cb (PlannerGanttRow *row)
+static gboolean 
+gantt_row_drag_item_to_pointer (PlannerGanttRow *row, gboolean scroll)
 {
-	GtkWidget *widget;
+	PlannerGanttChart *chart;
+	GnomeCanvas *canvas;
 	gint       width, height;
-	gint       x, y, dx = 0, dy = 0;
+	gint       x, y, item_cx, item_cy;
+	gint       cx, cy;
+	gdouble    wx2, wy2;
+	gint dx, dy;
+
+	gint         duration;
+	gint         work;
+	MrpProject  *project;
+	gchar       *message;
+
+	gint        percent_complete;
+
+	canvas = GNOME_CANVAS_ITEM (row)->canvas;
+	chart = g_object_get_data (G_OBJECT (GNOME_CANVAS_ITEM (row)->canvas), "chart");
 
-	widget = GTK_WIDGET (GNOME_CANVAS_ITEM (row)->canvas);
 	
 	/* Get the current mouse position so that we can decide if the pointer
 	 * is inside the viewport.
 	 */
-	gdk_window_get_pointer (widget->window, &x, &y, NULL);
+	gdk_window_get_pointer (GTK_WIDGET(canvas)->window, &x, &y, NULL);
 
-	width = widget->allocation.width;
-	height = widget->allocation.height;
+	gnome_canvas_get_scroll_offsets (canvas, &cx, &cy);
+
+	width = GTK_WIDGET(canvas)->allocation.width;
+	height = GTK_WIDGET(canvas)->allocation.height;
+
+	if(!scroll)
+	{
+		x = CLAMP(x, 0, width - 1);
+		y = CLAMP(y, 0, height - 1);
+	}
+
+	gnome_canvas_c2w (canvas, cx + x, cy + y, &wx2, &wy2);
+
+  	if(drag_item_wx_min != -1.0) wx2 = MAX(wx2, drag_item_wx_min);
+  	if(drag_item_wx_max != -1.0) wx2 = MIN(wx2, drag_item_wx_max);
+  	if(drag_item_wy_min != -1.0) wy2 = MAX(wy2, drag_item_wy_min);
+  	if(drag_item_wy_max != -1.0) wy2 = MIN(wy2, drag_item_wy_max);
+
+	switch(row->priv->state)
+	{
+	case STATE_DRAG_DURATION:
+
+		project = mrp_object_get_project (MRP_OBJECT (row->priv->task));
+
+		duration = MAX (0, (wx2 - row->priv->x_start) / row->priv->scale);
+
+		/* Snap to quarters. */
+		duration = floor (duration / SNAP + 0.5) * SNAP;
+
+		work = mrp_project_calculate_task_work (
+			project,
+			row->priv->task,
+			-1,
+			mrp_task_get_start (row->priv->task) + duration);
+
+		message = g_strdup_printf (_("Change work to %s"),
+					   planner_format_duration (project, work));
+		planner_gantt_chart_status_updated (chart, message);
+		g_free (message);
+
+		gnome_canvas_item_set (drag_item,
+				       "x2", wx2,
+				       NULL);
+		break;
+
+	case STATE_DRAG_COMPLETE:
+		percent_complete = floor ((CLAMP(wx2 - row->priv->x, 0, row->priv->width) * 100.0) / row->priv->width + 0.5);
+		message = g_strdup_printf (_("Change progress to %u%% complete"), percent_complete);
+		planner_gantt_chart_status_updated (chart, message);
+		g_free (message);
+
+		gnome_canvas_item_set (drag_item,
+				       "x2", wx2,
+				       NULL);
+		break;
+	case STATE_DRAG_LINK:
+		target_item = gnome_canvas_get_item_at (canvas, wx2, wy2);
+		
+		drag_points->coords[0] = drag_wx1;
+		drag_points->coords[1] = drag_wy1;
+		drag_points->coords[2] = wx2;
+		drag_points->coords[3] = wy2;
+	
+		gnome_canvas_item_set (drag_item,
+				       "points", drag_points,
+				       NULL);
+		
+		if (old_target_item && old_target_item != target_item) {
+			g_object_set (old_target_item,
+				      "highlight",
+				      FALSE,
+				      NULL);
+		}
+		
+		if (target_item && target_item != GNOME_CANVAS_ITEM(row)) {
+			const gchar *task_name, *target_name;
+			
+			g_object_set (target_item,
+				      "highlight",
+				      TRUE,
+				      NULL);
+
+			target_name = mrp_task_get_name (PLANNER_GANTT_ROW (target_item)->priv->task);
+			task_name = mrp_task_get_name (row->priv->task);
+			
+			if (target_name == NULL || target_name[0] == 0) {
+				target_name = _("No name");
+			}
+			if (task_name == NULL || task_name[0] == 0) {
+				task_name = _("No name");
+			}
+			
+			message = g_strdup_printf (_("Make task '%s' a predecessor of '%s'"),
+						   task_name,
+						   target_name);
+
+			planner_gantt_chart_status_updated (chart, message);
+
+			g_free (message);
+		}
+
+		if (target_item == NULL) {
+			planner_gantt_chart_status_updated (chart, NULL);
+		}
+		
+		old_target_item = target_item;
+		break;
+	default:
+		g_assert (FALSE);
+		break;
+	}
+
+	if(!scroll) return TRUE;
+
+	gnome_canvas_w2c (canvas, wx2, wy2, &item_cx, &item_cy);
 
 	if (x < 0) {
-		dx = x;
-	} else if (x >= widget->allocation.width) {
-		dx = x - widget->allocation.width + 1;
+		dx = MAX(x, (item_cx - cx) - width / 2);
+		dx = MIN(dx, 0);
+	} else if (x >= width) {
+		dx = MIN(x - width + 1, (item_cx - cx) - width/2);
+		dx = MAX(dx, 0);
 	} else {
 		dx = 0;
 	}
 
 	if (y < 0) {
 		dy = y;
-	} else if (y >= widget->allocation.height) {
-		dy = y - widget->allocation.height + 1;
+	} else if (y >= height) {
+		dy = y - height + 1;
 	} else {
 		dy = 0;
 	}
 	
-	gantt_row_canvas_scroll (widget, dx, dy);
-	
+	if(dx > 0)
+	{
+		planner_gantt_chart_reflow_now(chart);
+		gantt_row_canvas_scroll (GTK_WIDGET(canvas), dx, dy);
+	}
+	else if(dx < 0)
+	{
+		gantt_row_canvas_scroll (GTK_WIDGET(canvas), dx, dy);
+		planner_gantt_chart_reflow_now(chart);
+	}
+	else
+	{
+		if(dy != 0)
+		{
+			gantt_row_canvas_scroll (GTK_WIDGET(canvas), dx, dy);
+		}
+		planner_gantt_chart_reflow_now(chart);
+	}
+
 	return TRUE;
 }
 
-#define IN_DRAG_DURATION_SPOT(x,y,right,top,ymin,ymax)	\
-	((abs (x - (right)) <= 3) && \
-	 (y > top + ymin) && (y < top + ymax))
+static gboolean
+gantt_row_scroll_timeout_cb (PlannerGanttRow *row)
+{
+	return gantt_row_drag_item_to_pointer(row, TRUE);
+}
 
-#define IN_DRAG_RELATION_SPOT(x,y,right,top,ymin,ymax)	\
-	((x <= right) && \
-	 (y > top + ymin) && (y < top + ymax))
+static DragSpot 
+get_drag_spot(gdouble x, gdouble y, PlannerGanttRowPriv *priv)
+{
+	gdouble x2 = priv->x + priv->width;
+	if( (y > priv->y + priv->bar_top) &&
+	    (y < priv->y + priv->bar_bot) &&
+	    (x < x2 + FUZZ) ) {
+		gdouble complete_x2 = priv->x + floor(priv->width * (mrp_task_get_percent_complete (priv->task) / 100.0) + 0.5);
+
+		/* if not way left of end of completion bar */
+		if(x > complete_x2 - FUZZ) {
+
+			/* if end of completion bar and end of task bar are not very close together */
+			if(x2 - complete_x2 > 2 * FUZZ) {
+
+				/* if not way right of the completion bar */
+				if(x < complete_x2 + FUZZ) {
+					return DRAG_COMPLETE_SPOT;
+				} else { /* if way right of completion bar and ... */
+
+					/* if less than FUZZ from the end of the task bar */
+					if(x > x2 - FUZZ) {
+						return DRAG_DURATION_SPOT;
+					} else { /* if more than FUZZ left of the end of the task bar */
+						return DRAG_RELATION_SPOT;
+					}
+				}
+			} else { /* if DRAG_DURATION_SPOT and DRAG_COMPLETE_SPOT are connected and ... */
+
+				/* if closer to the end of the task bar than to the end of the completion bar */
+				if(x > complete_x2 + (x2 - complete_x2) / 2) {
+					return DRAG_DURATION_SPOT;
+				} else {
+					return DRAG_COMPLETE_SPOT;
+				}
+			}
+		} else { /* if way left of end of completion bar */
+			return DRAG_RELATION_SPOT;
+		}
+	} else { /* above, below or too far to the right of bar */
+		return DRAG_NONE_SPOT;
+	}
+}
 
 static gboolean
 gantt_row_event (GnomeCanvasItem *item, GdkEvent *event)
@@ -2320,19 +2533,14 @@ gantt_row_event (GnomeCanvasItem *item, GdkEvent *event)
 	PlannerGanttRowPriv      *priv;
 	PlannerGanttChart        *chart;
 	GtkWidget                *canvas_widget;
-	static gdouble            x1, y1;
 	gdouble                   wx1, wy1;
 	gdouble                   wx2, wy2;
-	static GnomeCanvasItem   *target_item;
-	static GnomeCanvasItem   *old_target_item;
-	static GnomeCanvasItem   *drag_item = NULL;
-	static GnomeCanvasPoints *drag_points = NULL;
 	MrpTask                  *task;
 	MrpTask                  *target_task;
 	GdkCursor                *cursor;
 	gboolean                  summary;
 	MrpTaskType               type;
-	gchar                    *message;
+	DragSpot                  drag_spot;
 	
 	row = PLANNER_GANTT_ROW (item);
 	priv = row->priv;
@@ -2345,9 +2553,9 @@ gantt_row_event (GnomeCanvasItem *item, GdkEvent *event)
 	case GDK_BUTTON_PRESS:
 		switch (event->button.button) {
 		case 3:
-			if (IN_DRAG_RELATION_SPOT (event->button.x, event->button.y,
-						   priv->x + priv->width, priv->y, 
-						   priv->bar_top, priv->bar_bot)) {
+			drag_spot = get_drag_spot(event->button.x, event->button.y, priv);
+
+			if (drag_spot == DRAG_RELATION_SPOT) {
 				PlannerGanttChart *chart;
 				PlannerTaskTree   *tree;
 				GtkTreePath       *path;
@@ -2395,13 +2603,10 @@ gantt_row_event (GnomeCanvasItem *item, GdkEvent *event)
 				break;
 			}
 
+			drag_spot = get_drag_spot(event->button.x, event->button.y, priv);
+
 			if (type != MRP_TASK_TYPE_MILESTONE &&
-			    !summary && IN_DRAG_DURATION_SPOT (event->button.x,
-							       event->button.y,
-							       priv->x + priv->width,
-							       priv->y,
-							       priv->bar_top, 
-							       priv->bar_bot)) {
+			    !summary && drag_spot == DRAG_DURATION_SPOT) {
 				guint rgba;
 				
 				priv->state = STATE_DRAG_DURATION;
@@ -2429,13 +2634,62 @@ gantt_row_event (GnomeCanvasItem *item, GdkEvent *event)
 									   "width_pixels", 1,
 									   NULL);
 					gnome_canvas_item_hide (drag_item);
+
+					/* set limits */
+					drag_item_wx_min = wx1;
+					drag_item_wx_max = -1.0;
+					drag_item_wy_min = wy1;
+					drag_item_wy_max = wy2;
 				}
-			} else if (IN_DRAG_RELATION_SPOT (event->button.x,
-							  event->button.y,
-							  priv->x + priv->width,
-							  priv->y,
-							  priv->bar_top, 
-							  priv->bar_bot)) {
+
+				/* Start the autoscroll timeout. */
+				priv->scroll_timeout_id = gtk_timeout_add (
+					50,
+					(GSourceFunc) gantt_row_scroll_timeout_cb,
+					row);
+			} else if (type != MRP_TASK_TYPE_MILESTONE &&
+			    !summary && drag_spot == DRAG_COMPLETE_SPOT) {
+				guint rgba;
+
+				priv->state = STATE_DRAG_COMPLETE;
+
+				wx1 = priv->x;
+				wy1 = priv->y + priv->bar_top + 4;
+				wx2 = event->button.x;
+				wy2 = priv->y + priv->bar_bot - 4;
+
+				gnome_canvas_item_i2w (item, &wx1, &wy1);
+				gnome_canvas_item_i2w (item, &wx2, &wy2);
+
+				/*      red            green          blue          alpha */
+				rgba = (0xb7 << 24) | (0xc3 << 16) | (0xc9 << 8) | (127 << 0);
+				
+				if (drag_item == NULL) {
+					drag_item = gnome_canvas_item_new (gnome_canvas_root (item->canvas),
+									   EEL_TYPE_CANVAS_RECT,
+									   "x1", wx1,
+									   "y1", wy1,
+									   "x2", wx2,
+									   "y2", wy2,
+									   "fill_color_rgba", rgba,
+									   "outline_color_rgba", 0,
+									   "width_pixels", 1,
+									   NULL);
+					gnome_canvas_item_hide (drag_item);
+
+					/* set limits */
+					drag_item_wx_min = wx1;
+					drag_item_wx_max = priv->x + priv->width;
+					drag_item_wy_min = wy1;
+					drag_item_wy_max = wy2;
+				}
+
+				/* Start the autoscroll timeout. */
+				priv->scroll_timeout_id = gtk_timeout_add (
+					50,
+					(GSourceFunc) gantt_row_scroll_timeout_cb,
+					row);
+			} else if (drag_spot == DRAG_RELATION_SPOT) {
 				priv->state = STATE_DRAG_LINK;
 				if (drag_points == NULL) {
 					drag_points = gnome_canvas_points_new (2);
@@ -2454,6 +2708,12 @@ gantt_row_event (GnomeCanvasItem *item, GdkEvent *event)
 								   NULL);
 				gnome_canvas_item_hide (drag_item);
 
+				/* set limits */
+				gnome_canvas_get_scroll_region(item->canvas, &drag_item_wx_min, 
+									     &drag_item_wy_min,
+									     &drag_item_wx_max,
+									     &drag_item_wy_max);
+
 				old_target_item = NULL;
 
 				/* Start the autoscroll timeout. */
@@ -2496,8 +2756,8 @@ gantt_row_event (GnomeCanvasItem *item, GdkEvent *event)
 						NULL,
 						event->button.time);
 
-			x1 = event->button.x;
-			y1 = event->button.y;
+			drag_wx1 = event->button.x;
+			drag_wy1 = event->button.y;
 
 			return TRUE;
 
@@ -2527,21 +2787,27 @@ gantt_row_event (GnomeCanvasItem *item, GdkEvent *event)
 			gint x, y;
 
 			gdk_window_get_pointer (event->motion.window, &x, &y, NULL);
-			gnome_canvas_c2w (item->canvas, x, y, &event->motion.x, &event->motion.y);
+			gnome_canvas_window_to_world(item->canvas, x, y, &event->motion.x, &event->motion.y);
 		}
 
 		if (!(priv->state & STATE_DRAG_ANY)) {
-			if (type != MRP_TASK_TYPE_MILESTONE &&
-			    !summary && IN_DRAG_DURATION_SPOT (event->button.x,
-							       event->button.y,
-							       priv->x + priv->width,
-							       priv->y,
-							       priv->bar_top, 
-							       priv->bar_bot)) {
-				cursor = gdk_cursor_new (GDK_RIGHT_SIDE);
-				gdk_window_set_cursor (canvas_widget->window, cursor);
-				if (cursor) {
-					gdk_cursor_unref (cursor);
+			drag_spot = get_drag_spot(event->button.x, event->button.y, priv);
+
+			if (type != MRP_TASK_TYPE_MILESTONE && !summary && 
+			    ( (drag_spot == DRAG_DURATION_SPOT) || (drag_spot == DRAG_COMPLETE_SPOT) ) ) {
+				if(drag_spot == DRAG_DURATION_SPOT) {
+					cursor = gdk_cursor_new (GDK_RIGHT_SIDE);
+					gdk_window_set_cursor (canvas_widget->window, cursor);
+					if (cursor) {
+						gdk_cursor_unref (cursor);
+					}
+				}
+				else { /* DRAG_COMPLETE_SPOT */
+					cursor = gdk_cursor_new (GDK_HAND2);
+					gdk_window_set_cursor (canvas_widget->window, cursor);
+					if (cursor) {
+						gdk_cursor_unref (cursor);
+					}
 				}
 			} else { /* Mouse over resource names (or short_name) ? */
  				gint res_index;
@@ -2565,103 +2831,11 @@ gantt_row_event (GnomeCanvasItem *item, GdkEvent *event)
 			}
 			return TRUE;
 		}
-		else if (priv->state == STATE_DRAG_LINK) {
-			target_item = gnome_canvas_get_item_at (item->canvas,
-								event->motion.x,
-								event->motion.y);
-			
-			gnome_canvas_item_raise_to_top (drag_item);
-			gnome_canvas_item_show (drag_item);
-			
-			drag_points->coords[0] = x1;
-			drag_points->coords[1] = y1;
-			drag_points->coords[2] = event->motion.x;
-			drag_points->coords[3] = event->motion.y;
-		
-			gnome_canvas_item_set (drag_item,
-					       "points", drag_points,
-					       NULL);
-			
-			chart = g_object_get_data (G_OBJECT (item->canvas),
-						   "chart");
-			
-			if (old_target_item && old_target_item != target_item) {
-				g_object_set (old_target_item,
-					      "highlight",
-					      FALSE,
-					      NULL);
-			}
-			
-			if (target_item && target_item != item) {
-				const gchar *task_name, *target_name;
-				
-				g_object_set (target_item,
-					      "highlight",
-					      TRUE,
-					      NULL);
-
-				target_name = mrp_task_get_name (PLANNER_GANTT_ROW (target_item)->priv->task);
-				task_name = mrp_task_get_name (priv->task);
-				
-				if (target_name == NULL || target_name[0] == 0) {
-					target_name = _("No name");
-				}
-				if (task_name == NULL || task_name[0] == 0) {
-					task_name = _("No name");
-				}
-				
-				message = g_strdup_printf (_("Make task '%s' a predecessor of '%s'"),
-							   task_name,
-							   target_name);
-
-				planner_gantt_chart_status_updated (chart, message);
-
-				g_free (message);
-			}
-
-			if (target_item == NULL) {
-				planner_gantt_chart_status_updated (chart, NULL);
-			}
-			
-			old_target_item = target_item;
-		}
-		else if (priv->state == STATE_DRAG_DURATION) {
-			gint         duration;
-			gint         work;
-			MrpProject  *project;
-
-			project = mrp_object_get_project (MRP_OBJECT (priv->task));
-
-			wx2 = event->motion.x;
-			wy2 = priv->y + priv->bar_bot;
-			
-			gnome_canvas_item_i2w (item, &wx2, &wy2);
-
-			gnome_canvas_item_set (drag_item,
-					       "x2", wx2,
-					       "y2", wy2,
-					       NULL);
-			
+		else {
 			gnome_canvas_item_raise_to_top (drag_item);
 			gnome_canvas_item_show (drag_item);
 
-			chart = g_object_get_data (G_OBJECT (item->canvas), "chart");
-
-			duration = MAX (0, (event->motion.x - priv->x_start) / priv->scale);
-
-			/* Snap to quarters. */
-			duration = floor (duration / SNAP + 0.5) * SNAP;
-
-			work = mrp_project_calculate_task_work (
-				project,
-				priv->task,
-				-1,
-				mrp_task_get_start (priv->task) + duration);
-
-			message = g_strdup_printf (_("Change work to %s"),
-						   planner_format_duration (project, work));
-			planner_gantt_chart_status_updated (chart, message);
-			g_free (message);
+			gantt_row_drag_item_to_pointer (row, FALSE);
 		}
 			
 		break;
@@ -2709,6 +2883,11 @@ gantt_row_event (GnomeCanvasItem *item, GdkEvent *event)
 						"work",
 						&value);
 
+			if (priv->scroll_timeout_id) {
+				g_source_remove (priv->scroll_timeout_id);
+				priv->scroll_timeout_id = 0;
+			}
+
 			/*g_object_set (priv->task,
 				      "work", work,
 				      NULL);
@@ -2718,6 +2897,32 @@ gantt_row_event (GnomeCanvasItem *item, GdkEvent *event)
 
 			planner_gantt_chart_status_updated (chart, NULL);
 		}
+		else if (priv->state == STATE_DRAG_COMPLETE) {
+			GValue      value = { 0 };
+			gint        percent_complete = floor((MIN(MAX(0, event->button.x - priv->x), priv->width) * 100.0) / priv->width + 0.5);
+
+			chart = g_object_get_data (G_OBJECT (item->canvas), "chart");
+			tree = planner_gantt_chart_get_view (chart);
+
+			g_value_init (&value, G_TYPE_INT);
+			g_value_set_int (&value, percent_complete);
+
+			task_cmd_edit_property (planner_task_tree_get_window (tree),
+						tree,
+						priv->task,
+						"percent_complete", 
+						&value);
+
+			if (priv->scroll_timeout_id) {
+				g_source_remove (priv->scroll_timeout_id);
+				priv->scroll_timeout_id = 0;
+			}
+
+			gtk_object_destroy (GTK_OBJECT (drag_item));
+			drag_item = NULL;
+
+			planner_gantt_chart_status_updated (chart, NULL);
+		}
 		else if (priv->state == STATE_DRAG_LINK) {
 			if (old_target_item) {
 				g_object_set (old_target_item,
@@ -2835,9 +3040,8 @@ gantt_row_event (GnomeCanvasItem *item, GdkEvent *event)
 		
 	case GDK_2BUTTON_PRESS:
 		if (event->button.button == 1) {
-			if (IN_DRAG_RELATION_SPOT (event->button.x, event->button.y,
-						   priv->x + priv->width, priv->y, 
-						   priv->bar_top, priv->bar_bot)) {
+			drag_spot = get_drag_spot(event->button.x, event->button.y, priv);
+			if (drag_spot == DRAG_RELATION_SPOT) {
 				PlannerTaskTree  *tree;
 				GtkTreePath      *path;
 				GtkTreeSelection *selection;
