@@ -21,19 +21,15 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifndef __MG_TASK_INPUT_DIALOG_H__
-#define __MG_TASK_INPUT_DIALOG_H__
+#ifndef __PLANNER_TASK_INPUT_DIALOG_H__
+#define __PLANNER_TASK_INPUT_DIALOG_H__
 
 #include <gtk/gtkwidget.h>
-#include <libplanner/mrp-project.h>
-
-GtkWidget * planner_task_input_dialog_new (MrpProject *project);
-
-#endif /* __MG_TASK_INPUT_DIALOG_H__ */
-
-
+#include "planner-window.h"
 
+GtkWidget * planner_task_input_dialog_new (PlannerWindow *main_window);
 
+#endif /* __PLANNER_TASK_INPUT_DIALOG_H__ */
 
 
 
